import logging
from dataclasses import dataclass

from boto3.s3.transfer import TransferConfig

from yes3.utils.logs import check_level, get_logger

PROGRESS_MODES = {'off', 'all', 'large'}


@dataclass
class YeS3Config:
    default_region: str = 'us-east-2'
    log_level: int = logging.WARNING
    progress_mode: str = 'large'
    progress_size_threshold: int | float = 100e6  # bytes
    upload_transfer_config: TransferConfig = TransferConfig()

    @staticmethod
    def check_progress_mode(value) -> str:
        if value is None:
            value = 'off'
        if not isinstance(value, str):
            raise TypeError(f'progress_mode must be a str with one of the following values: {PROGRESS_MODES}')
        elif value.lower() not in PROGRESS_MODES:
            raise ValueError(f"Invalid progress_mode '{value}', must be one of {PROGRESS_MODES}")
        return value.lower()

    def __setattr__(self, name, value):
        if name == 'progress_mode':
            value = self.check_progress_mode(value)
        if name == 'log_level':
            root_logger = get_logger()
            value = check_level(value)
            if value != root_logger.level:
                root_logger.setLevel(value)
        super().__setattr__(name, value)



import argparse
import pdb
import sys
import traceback
import unittest
from typing import Type, Union


def get_arg_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='store_true', help='Run unittest.main with verbosity==2')
    parser.add_argument('-s', '--step', action='store_true', help='Step through with pdb.')
    parser.add_argument('--pdb', action='store_true', help='Enter pdb.post_mortem on failure.')
    return parser


def run_tests(args, test_case: Union[unittest.TestCase, Type[unittest.TestCase]], with_unittest=True) -> None:
    if isinstance(args, argparse.ArgumentParser):
        args = args.parse_args()
    if isinstance(test_case, type):
        test_case = test_case()
    if args.pdb or args.step or not with_unittest:
        try:
            if args.step:
                pdb.set_trace()
            tests = [attr for attr in dir(test_case) if callable(getattr(test_case, attr)) and attr.startswith('test_')]
            for test in tests:
                print(f"TESTING: {test}()... ", end='')
                getattr(test_case, test)()
                print("PASSED")
        except (KeyboardInterrupt, pdb.bdb.BdbQuit):
            sys.exit(1)
        except Exception:
            print("FAILED")
            traceback.print_exc()
            if args.pdb:
                pdb.post_mortem()
    else:
        unittest.main(verbosity=(args.verbose + 1))



from typing import Optional, Self

import boto3
from botocore.client import BaseClient
from botocore.session import Session


class ClientContainer:
    special_attrs = {'special_attrs', 'configure', 'client', 'session'}

    def __init__(self):
        self.client: Optional[BaseClient] = None
        self.session: Optional[Session] = None

    def configure(
        self,
        client: Optional[BaseClient] = None,
        region: Optional[str] = None,
        profile: Optional[str] = None,
        access_key: Optional[str] = None,
        secret_access_key: Optional[str] = None,
        session_token: Optional[str] = None,
    ) -> Self:
        if client is not None:
            self.client = client
        else:
            sess_kwargs = {}
            client_kwargs = {}
            if profile:
                sess_kwargs['profile_name'] = profile
            if access_key:
                sess_kwargs['aws_access_key_id'] = access_key
            if secret_access_key:
                sess_kwargs['aws_secret_access_key'] = secret_access_key
            if session_token:
                sess_kwargs['aws_session_token'] = session_token
            if region:
                client_kwargs['region_name'] = region
            if sess_kwargs:
                self.session = boto3.Session(**sess_kwargs)
                self.client = self.session.client('s3', **client_kwargs)
            else:
                self.client = boto3.client('s3', **client_kwargs)
        return self

    def __getattribute__(self, name: str):
        if name in type(self).special_attrs:
            return super().__getattribute__(name)
        else:
            if self.client is None:
                self.configure()
            return getattr(self.client, name)


_client = ClientContainer()


def configure_client(
        client: Optional[BaseClient] = None,
        region: Optional[str] = None,
        profile: Optional[str] = None,
        access_key: Optional[str] = None,
        secret_access_key: Optional[str] = None,
        session_token: Optional[str] = None,
) -> ClientContainer:
    return _client.configure(
        client=client,
        region=region,
        profile=profile,
        access_key=access_key,
        secret_access_key=secret_access_key,
        session_token=session_token
    )


def get_client() -> ClientContainer:
    return _client


def set_client(client: ClientContainer):
    global _client
    _client = client
